//
//  AWViewController.m
//  OverdueAssignment
//
//  Created by Andrew Weiss on 11/7/13.
//  Copyright (c) 2013 Andrew Weiss. All rights reserved.
//

#import "AWViewController.h"

@interface AWViewController ()

@end

@implementation AWViewController

#pragma mark - Default UIViewController Methods

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    
    NSArray *taskPropertyListObjects = [[NSUserDefaults standardUserDefaults] objectForKey:TASK_OBJECTS_ARRAY_KEY];
    for (NSDictionary *task in taskPropertyListObjects) {
        [self.taskObjects addObject:[self propertyListAsTaskObject:task]];
    }
    
    [self.tableView reloadData];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Lazy Instantiation

- (NSMutableArray *)taskObjects
{
    if (!_taskObjects) _taskObjects = [[NSMutableArray alloc] init];
    return _taskObjects;
}

#pragma mark - UIBarButtonItem IBActions

- (IBAction)addTaskBarButtonItemPressed:(UIBarButtonItem *)sender
{
    [self.tableView setEditing:NO animated:YES];
    [self performSegueWithIdentifier:@"toAWAddTaskViewControllerSegue" sender:sender];
}

- (IBAction)reorderBarButtonItemPressed:(UIBarButtonItem *)sender
{
    if (!self.tableView.editing) [self.tableView setEditing:YES animated:YES];
    else [self.tableView setEditing:NO animated:YES];
}

#pragma mark - UITableViewDelegate/UITableViewDataSource Methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [self.taskObjects count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"TaskItemCell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    
    if (indexPath.section == 0) {
        AWTask *task = self.taskObjects[indexPath.row];
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateFormat:@"MM-dd-yyyy"];
        cell.textLabel.text = task.title;
        cell.detailTextLabel.text = [@"Due Date: " stringByAppendingString:[dateFormatter stringFromDate:task.date]];
        
        cell.backgroundColor =  ([self isDateGreaterThanDate:[NSDate date] and:task.date]) ? [UIColor redColor] : [UIColor yellowColor];
        
        if ([self.taskObjects[indexPath.row] completion]) cell.backgroundColor = [UIColor greenColor];
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [self updateCompletionOfTask:self.taskObjects[indexPath.row] forIndexPath:indexPath];
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [self.taskObjects removeObjectAtIndex:indexPath.row];
        NSMutableArray *taskObjectsFromNSUserDefaults = [[[NSUserDefaults standardUserDefaults] arrayForKey:TASK_OBJECTS_ARRAY_KEY] mutableCopy];
        [taskObjectsFromNSUserDefaults removeObjectAtIndex:indexPath.row];
        [[NSUserDefaults standardUserDefaults] setObject:taskObjectsFromNSUserDefaults forKey:TASK_OBJECTS_ARRAY_KEY];
        [[NSUserDefaults standardUserDefaults] synchronize];
        
        [self.tableView reloadData];
    }
}

- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath
{
    [self performSegueWithIdentifier:@"toAWDetailTaskViewControllerSegue" sender:indexPath];
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    return (tableView.editing) ? YES : NO;
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath
{
    AWTask *selectedTask = self.taskObjects[sourceIndexPath.row];
    [self.taskObjects removeObjectAtIndex:sourceIndexPath.row];
    [self.taskObjects insertObject:selectedTask atIndex:destinationIndexPath.row];
    
    [[NSUserDefaults standardUserDefaults] setObject:[self saveTasks] forKey:TASK_OBJECTS_ARRAY_KEY];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
}

#pragma mark - Helper Methods

- (NSDictionary *)taskObjectAsPropertyList:(AWTask *)taskObject
{
    NSDictionary *taskPropertyList = @{TITLE : taskObject.title, DESCRIPTION : taskObject.description, DATE : taskObject.date, COMPLETION : @(taskObject.completion)};
    return taskPropertyList;
}

- (AWTask *)propertyListAsTaskObject:(NSDictionary *)taskPropertyListItem
{
    AWTask *task = [[AWTask alloc] initWithData:taskPropertyListItem];
    return task;
}

- (void)updateCompletionOfTask:(AWTask *)task forIndexPath:(NSIndexPath *)indexPath
{
    NSMutableArray *tasksFromUserDefaults = [[[NSUserDefaults standardUserDefaults] arrayForKey:TASK_OBJECTS_ARRAY_KEY] mutableCopy];
    [tasksFromUserDefaults removeObjectAtIndex:indexPath.row];
    
    task.completion = (!task.completion) ? YES : NO;
    [tasksFromUserDefaults insertObject:[self taskObjectAsPropertyList:task] atIndex:indexPath.row];
    
    [[NSUserDefaults standardUserDefaults] setObject:tasksFromUserDefaults forKey:TASK_OBJECTS_ARRAY_KEY];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    [self.tableView reloadData];
}

- (BOOL)isDateGreaterThanDate:(NSDate *)date and:(NSDate *)toDate
{
    if ([date timeIntervalSince1970] > [toDate timeIntervalSince1970]) {
        return YES;
    }
    
    return NO;
}

- (NSMutableArray *)saveTasks
{
    NSMutableArray *tasks = [[NSMutableArray alloc] init];
    for (AWTask *task in self.taskObjects) {
        NSDictionary *taskAsDictionary = [self taskObjectAsPropertyList:task];
        [tasks addObject:taskAsDictionary];
    }
    
    return tasks;
}

#pragma mark - AWAddTaskViewControllerDelegate Methods

- (void)didAddTask:(AWTask *)task
{
    [self.taskObjects addObject:task];
    NSMutableArray *storedTaskObjects = [[[NSUserDefaults standardUserDefaults] arrayForKey:TASK_OBJECTS_ARRAY_KEY] mutableCopy];
    if (!storedTaskObjects) storedTaskObjects = [[NSMutableArray alloc] init];
    [storedTaskObjects addObject:[self taskObjectAsPropertyList:task]];
    [[NSUserDefaults standardUserDefaults] setObject:storedTaskObjects forKey:TASK_OBJECTS_ARRAY_KEY];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    [self.navigationController dismissViewControllerAnimated:YES completion:nil];
    
    [self.tableView reloadData];
}

- (void)didCancel
{
    [self.navigationController dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - AWDetailTaskViewControllerDelegate Methods

- (void)didFinishEditingTask:(AWTask *)task atIndexPath:(NSIndexPath *)indexPath
{
    [self.taskObjects replaceObjectAtIndex:indexPath.row withObject:task];
    [[NSUserDefaults standardUserDefaults] setObject:[self saveTasks] forKey:TASK_OBJECTS_ARRAY_KEY];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    [self.tableView reloadData];
}

#pragma mark - Navigation and Segues

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([sender isKindOfClass:[UIBarButtonItem class]] && [segue.destinationViewController isKindOfClass:[AWAddTaskViewController class]]) {
        AWAddTaskViewController *addTaskViewController = segue.destinationViewController;
        addTaskViewController.delegate = self;
    }
    else if ([sender isKindOfClass:[NSIndexPath class]] && [segue.destinationViewController isKindOfClass:[AWDetailTaskViewController class]]) {
        AWDetailTaskViewController *detailTaskViewController = segue.destinationViewController;
        NSIndexPath *selectedTaskIndexPath = sender;
        detailTaskViewController.selectedTask = self.taskObjects[selectedTaskIndexPath.row];
        detailTaskViewController.delegate = self;
    }
}

@end